##-*- Mode: CPerl -*-

## File: Pd/Patch/obj.pm
## Author: Bryan Jurish <moocow@ling.uni-potsdam.de>
## Description:
##  + class for Pd canvas objects
##======================================================================

use strict;

##======================================================================
## obj: generic
package Pd::Patch::X_obj;
use Pd::Patch::Item;
our @ISA = qw(Pd::Patch::Item);
sub PROPERTIES { return [qw(decl type x y name args)]; }
sub DEFAULTS   { return {decl=>'#X', type=>'obj', 'x'=>0, 'y'=>0, name=>'0'}; }
__PACKAGE__->init();

##-- aliases
@Pd::Patch::obj::ISA    = (__PACKAGE__);
@Pd::Patch::Object::ISA = (__PACKAGE__);

##-- new()
##   + dispatch to class prefixes of ${name}:
##     Pd::Patch::, Pd::Patch::Item::, Pd::Patch::obj::, Pd::Patch::Object::
sub new {
  my $that = shift;
  ##
  ##-- get pd name: check for pd subclasses
  my $name = (grep {!ref($_) || ref($_) ne 'HASH'} @_)[$that->PROPERTIES_byname->{name}];
  my $thatclass = ref($that)||$that;
  my ($class);
  if (defined($name)) {
    my $suffix = $name;
    $suffix =~ s/\#//g;
    my @prefixes = ("Pd::Patch::", "Pd::Patch::Item::", "Pd::Patch::obj::", "Pd::Patch::Object::");
    my ($prefix);
    foreach $prefix (@prefixes) {
      $class = $prefix.$suffix;
      return $class->new(@_)
	if (UNIVERSAL::isa($class,$thatclass) && $class ne $thatclass);
    }
  }
  return $thatclass->_new(@_);
}

1; ##-- be happy

__END__

##======================================================================
## Docs
=pod

=head1 NAME

Pd::Patch - Perl parser and wrapper classes for Pd abstractions

=head1 SYNOPSIS

 use Pd::Patch;

 #... stuff happens...

=cut

##======================================================================
## Description
=pod

=head1 DESCRIPTION

not yet written

=cut

##======================================================================
## Footer
=pod

=head1 ACKNOWLEDGEMENTS

perl by Larry Wall.

Pd by Miller Puckette and others.

=head1 AUTHOR

Bryan Jurish E<lt>moocow@ling.uni-potsdam.deE<gt>

=head1 COPYRIGHT

Copyright (c) 2007, Bryan Jurish.  All rights reserved.

This package is free software.  You may redistribute it
and/or modify it under the same terms as Perl itself.

=head1 SEE ALSO

perl(1),
pd(1).

=cut
